/*
  Java gfx-test framework

   Copyright (C) 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.testsuites;

import java.awt.BasicStroke;
import java.awt.Graphics2D;



import org.gfxtest.callbacks.CommonCircleDrawCallbacks;
import org.gfxtest.framework.ColorPalette;
import org.gfxtest.framework.GrayscalePalette;
import org.gfxtest.framework.HSBPalette;
import org.gfxtest.framework.PrintTest;
import org.gfxtest.framework.TestImage;
import org.gfxtest.framework.TestResult;
import org.gfxtest.framework.annotations.GraphicsPrimitive;
import org.gfxtest.framework.annotations.GraphicsPrimitives;
import org.gfxtest.framework.annotations.RenderStyle;
import org.gfxtest.framework.annotations.RenderStyles;
import org.gfxtest.framework.annotations.TestType;
import org.gfxtest.framework.annotations.TestTypes;
import org.gfxtest.framework.annotations.Transformation;
import org.gfxtest.framework.annotations.Transformations;
import org.gfxtest.framework.annotations.Zoom;



/**
 * This test suite checks rendering of various circles.
 * 
 * @author Pavel Tisnovsky
 */
@TestType(TestTypes.PRINT_TEST)
@GraphicsPrimitive(GraphicsPrimitives.CIRCLE)
@RenderStyle(RenderStyles.NORMAL)
@Transformation(Transformations.NONE)
@Zoom(1)
public class PrintTestCircles extends PrintTest
{

    /**
     * Method which renders set of circles using various colors and
     * stroke styles. For each circle, the callback function/method is called to
     * perform all required setup.
     * 
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @param radiusStep
     *            between two near circles
     * @param circleDrawCallback
     *            class containing set of callback methods
     */
    private void drawCircle(TestImage image, Graphics2D graphics2d, int radiusStep, CommonCircleDrawCallbacks circleDrawCallback)
    {
        // setup rendering
        circleDrawCallback.setup(image, graphics2d);

        // image width and height
        final int width = image.getWidth();
        final int height = image.getHeight();

        // horizontal coordinates of circle center
        final int xc = width >> 1;
        final int yc = height >> 1;

        // maximum radius
        final int maxRadius = Math.min(width, height) - BORDER;

        // index to color palette
        int colorIndex = 0;

        // draw all circles onto a paper
        for (int radius = MINIMUM_RADIUS; radius < maxRadius; radius += radiusStep)
        {
            // setup can be made for each circle
            circleDrawCallback.iterationCallBack(xc, yc, radius, maxRadius, colorIndex++);
            // render the circle
            graphics2d.drawOval(xc - radius, yc - radius, radius << 1, radius << 1);
        }

        // cleanup rendering
        circleDrawCallback.cleanup();
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleBasicStyle(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP, new CommonCircleDrawCallbacks()
        {
            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // nothing needs to be changed or set
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles color are selected from a palette.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleColorPalette(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP, new CommonCircleDrawCallbacks()
        {
            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set new circle color to index
                this.graphics.setColor(ColorPalette.getColor(index));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles color are selected from a palette.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleColorPaletteInv(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP, new CommonCircleDrawCallbacks()
        {
            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set new circle color according to index
                this.graphics.setColor(ColorPalette.getColor(MAX_COLOR_INDEX - index));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles color are selected from a hue palette.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleColorScale(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP, new CommonCircleDrawCallbacks()
        {
            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // compute hue value
                float hue = radius * 1.0f / maxRadius;
                // set circle color
                this.graphics.setColor(HSBPalette.createHsbColor(hue));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles color are selected from a hue palette.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleColorScaleInv(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP, new CommonCircleDrawCallbacks()
        {
            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // compute hue value
                float hue = 1.0f - radius * 1.0f / maxRadius;
                // set circle color
                this.graphics.setColor(HSBPalette.createHsbColor(hue));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles color are selected from a grayscale palette.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleGrayScale(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP, new CommonCircleDrawCallbacks()
        {
            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // compute grayscale value
                float gray = radius * 1.0f / maxRadius;
                // clamp the grayscale value
                if (gray > 1.0f)
                {
                    gray = 1.0f;
                }
                // set circle color
                this.graphics.setColor(GrayscalePalette.createGrayscaleColor(gray));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles color are selected from a grayscale palette.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleGrayScaleInv(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP, new CommonCircleDrawCallbacks()
        {
            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // compute grayscale value
                float gray = 1.0f - radius * 1.0f / maxRadius;
                // clamp the grayscale value
                if (gray < 0.0f)
                {
                    gray = 0.0f;
                }
                // set circle color
                this.graphics.setColor(GrayscalePalette.createGrayscaleColor(gray));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and default end caps.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidth(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width
                this.graphics.setStroke(new BasicStroke(this.strokeWidth));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_BUTT.
     * Join style is set to bevel style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapButtJoinBevel(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_ROUND.
     * Join style is set to bevel style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapRoundJoinBevel(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_SQUARE.
     * Join style is set to bevel style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapSquareJoinBevel(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_BUTT.
     * Join style is set to miter style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapButtJoinMiter(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_ROUND.
     * Join style is set to miter style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapRoundJoinMiter(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_SQUARE.
     * Join style is set to miter style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapSquareJoinMiter(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_BUTT.
     * Join style is set to round style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapButtJoinRound(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_ROUND));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_ROUND.
     * Join style is set to round style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapRoundJoinRound(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_SQUARE.
     * Join style is set to round style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapSquareJoinRound(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_ROUND));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_BUTT.
     * Join style is set to bevel style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapButtJoinBevelInv(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_ROUND.
     * Join style is set to bevel style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapRoundJoinBevelInv(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_SQUARE.
     * Join style is set to bevel style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapSquareJoinBevelInv(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_BUTT.
     * Join style is set to miter style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapButtJoinMiterInv(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_ROUND.
     * Join style is set to miter style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapRoundJoinMiterInv(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_SQUARE.
     * Join style is set to miter style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapSquareJoinMiterInv(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_BUTT.
     * Join style is set to round style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapButtJoinRoundInv(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_ROUND));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_ROUND.
     * Join style is set to round style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapRoundJoinRoundInv(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and end caps set to CAP_SQUARE.
     * Join style is set to round style.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthCapSquareJoinRoundInv(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_ROUND));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles are rendered with various width and default end caps.
     * Color of all rendered circles are set to black.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleChangeWidthInv(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP << 1, new CommonCircleDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth));
                // and change it for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // decrease stroke width
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Entry point to the test suite.
     * 
     * @param args
     *            not used in this case
     */
    public static void main(String[] args)
    {
        new PrintTestCircles().runTestSuite(args);
    }
}
